import ctypes
import threading


class Library(object):
    def __init__(self, ctypes_library):
        self._func_lock = threading.Lock()
        self._library = ctypes_library
        # We cache the cfunc object from the ctypes.CDLL object
        self.cfuncs = {}
        self.function_args = {}

        # Definition from FlexrioI2CBusInstance.h
        self.i2c_bus_instance = ctypes.c_char * 16

        session_type = ctypes.c_uint32

        self.function_args['niFlexRIO_OpenSession'] = [
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_int32,
            ctypes.POINTER(session_type)
        ]

        self.function_args['niFlexRIO_CloseSession'] = [
            session_type,
            ctypes.c_int32,
        ]

        self.function_args['niFlexRIO_GetError'] = [
            session_type,
            ctypes.c_int32,
            ctypes.c_size_t,
            ctypes.c_char_p,
        ]

        self.function_args['niFlexRIOMDK_readIoModuleInfo'] = [
            session_type,                           # session
            ctypes.POINTER(ctypes.c_uint16),        # product id
            ctypes.POINTER(ctypes.c_uint16),        # vendor id
            ctypes.POINTER(ctypes.c_uint32),        # serial number
            ctypes.POINTER(ctypes.c_uint32),        # part number
            ctypes.POINTER(ctypes.c_uint8),         # variant info
            ctypes.POINTER(ctypes.c_uint16),        # hardware revision
        ]

        self.function_args['niFlexRIO_WriteI2C'] = [
            session_type,                           # session
            ctypes.c_char_p,                        # bus instance
            ctypes.c_uint16,                        # slave address
            ctypes.POINTER(ctypes.c_uint8),         # data
            ctypes.c_size_t,                        # num bytes to write
            ctypes.c_uint8,                         # repeated start
            ctypes.POINTER(ctypes.c_size_t),        # num bytes written
        ]

        self.function_args['niFlexRIO_TransferI2C'] = [
            session_type,                           # session
            ctypes.c_char_p,                        # bus instance
            ctypes.c_uint16,                        # slave address
            ctypes.POINTER(ctypes.c_uint8),         # write data
            ctypes.c_size_t,                        # num bytes to write
            ctypes.POINTER(ctypes.c_uint8),         # read data
            ctypes.c_size_t,                        # num bytes to read
            ctypes.c_uint8,                         # repeated start
            ctypes.POINTER(ctypes.c_size_t),        # num bytes written
            ctypes.POINTER(ctypes.c_size_t),        # num bytes read
        ]

    def __getattr__(self, attr):
        with self._func_lock:
            if attr not in self.cfuncs:
                self.cfuncs[attr] = self._library[attr]
                self.cfuncs[attr].argtypes = self.function_args[attr]
                self.cfuncs[attr].restype = ctypes.c_int32
        return self.cfuncs[attr]

    def __getitem__(self, item):
        return self.__getattr__(item)
