import ctypes
import library
import platform
import threading


class Error(Exception):
    def __init__(self, message):
        super(Error, self).__init__(message)


class UnsupportedConfigurationError(Error):
    def __init__(self):
        super(UnsupportedConfigurationError, self).__init__('System configuration is unsupported: ' + platform.architecture()[0] + ' ' + platform.system())


class DriverNotInstalledError(Error):
    def __init__(self):
        super(DriverNotInstalledError, self).__init__('The NI-FlexRIO runtime could not be loaded. Make sure it is installed and its bitness matches that of your Python interpreter. Please visit http://www.ni.com/downloads/drivers/ to download and install it.')


_instance = None
_instance_lock = threading.Lock()
_library_info = {'Linux': {'64bit': {'name': 'libniflexrioapi.so', 'type': 'cdll'}},
                 'Windows': {'32bit': {'name': 'niflexrioapi.dll', 'type': 'cdll'},
                             '64bit': {'name': 'niflexrioapi.dll', 'type': 'cdll'}}}


def _get_library_name():
    try:
        return _library_info[platform.system()][platform.architecture()[0]]['name']
    except KeyError:
        raise UnsupportedConfigurationError


def _get_library_type():
    try:
        return _library_info[platform.system()][platform.architecture()[0]]['type']
    except KeyError:
        raise UnsupportedConfigurationError


def get():
    global _instance
    global _instance_lock

    with _instance_lock:
        if _instance is None:
            try:
                library_type = _get_library_type()
                if library_type == 'windll':
                    ctypes_library = ctypes.WinDLL(_get_library_name())
                else:
                    assert library_type == 'cdll'
                    ctypes_library = ctypes.CDLL(_get_library_name())
            except OSError:
                raise DriverNotInstalledError()
            _instance = library.Library(ctypes_library)
        return _instance
