﻿import os
from serdesconfig import flexrio_wrapper
from serdesconfig import maxim_adapter
from serdesconfig import ti_adapter
import sys
import time  # need this for PXIe-1486 scripts that use time and don't import it


def _parse_args(argv):
    import argparse
    parser = argparse.ArgumentParser(description='NI-FlexRIO Command Line Utility for PXIe-1486 and PXIe-1487 CSI-2 serializers, deserializers, and camera modules.',
                                     formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('resource',
                        help='the name of the RIO resource to use for configuration')
    parser.add_argument('bitfile',
                        help='the path of the bitfile to use for configuration')
    parser.add_argument('serdes',
                        help='the name of the i2c bus instance to use for configuration')
    parser.add_argument('script',
                        help='the configuration script to execute')
    parser.add_argument('-s', '--simulated_module',
                        help='enable execution of utility without opening a FlexRIO session for testing script syntax',
                        default=0,  # if the option isn't specified, use 0 for simulated_module value
                        const=flexrio_wrapper.ModelName.PXIE_1486,  # if -s is specified with no argument, default to NI PXIe-1486
                        choices=[e for e in flexrio_wrapper.ModelName().models()],
                        action='store',
                        nargs='?',
                        type=int)
    parser.add_argument('-v', '--verbose',
                        help='increase output verbosity',
                        action='store_true')
    parser.add_argument('-w', '--wait',
                        help='wait for keypress',
                        action='store_true')
    parser.add_argument('-p', '--posc',
                        help='create posc config binary file',
                        type=argparse.FileType('wb'))
    return parser.parse_args(argv)


def execute_script_ti(fw, script):
    if not script.endswith('.py'):
        raise ValueError('script must have .py file format')
    board = ti_adapter.Board(fw)
    # change to script dir to accommodate scripts referencing relative paths
    orig_cwd = os.getcwd()
    os.chdir(os.path.dirname(os.path.abspath(script)))
    # launch configuration script in the same process to leverage our board object
    exec(open(script).read())
    os.chdir(orig_cwd)


def execute_script_maxim(fw, script):
    if not script.endswith('.cpp'):
        raise ValueError('script must have .cpp file format')
    maxim_adapter.execute_script(fw, script)


def launch_script(fw, script):
    fw.pre_execute(script)
    if fw.serdes_vendor == flexrio_wrapper.SerDesVendor.TI:
        execute_script_ti(fw, script)
    else:
        execute_script_maxim(fw, script)
    fw.post_execute()


def main(argv=sys.argv[1:]):
    args = _parse_args(argv)

    if args.wait:
        raw_input('Press Enter to continue...')

    fw = flexrio_wrapper.FlexRIOWrapper(args.resource, args.bitfile, args.serdes, args.simulated_module, args.script, args.posc, args.verbose)
    launch_script(fw, args.script)


if __name__ == '__main__':
    main()
