import flexrio_wrapper


class Board(object):
    def __init__(self, fw):
        self.fw = fw
        self.devAddr = 0x30 if self.fw.serdes_type == flexrio_wrapper.SerDesType.SER else 0x60

    def WriteI2C(self, dev_addr, reg_addr, values):
        bytes_to_write = bytearray([reg_addr, ])
        if isinstance(values, list) or isinstance(values, bytearray):
            if (len(values) > 0):
                bytes_to_write.extend(values)
            else:
                raise ValueError('values parameter must have at least one value')
        else:
            bytes_to_write.append(values)
        self.fw.write_i2c(dev_addr, bytes_to_write)

    def ReadI2C(self, dev_addr, reg_addr, reg_len=1):
        bytes_to_write = bytearray([reg_addr, ])
        num_bytes_to_read = 0
        if isinstance(reg_len, list) or isinstance(reg_len, bytearray):
            if (len(reg_len) > 1):
                bytes_to_write.extend(reg_len[:-1])
                num_bytes_to_read = reg_len[-1]
            elif (len(reg_len) > 0):
                bytes_to_write.extend(reg_len)
                num_bytes_to_read = 1
            else:
                raise ValueError('reg_len list must have at least one value')
        else:
            num_bytes_to_read = reg_len

        return self.fw.transfer_i2c(dev_addr, bytes_to_write, num_bytes_to_read)

    def WriteReg(self, reg_addr, values):
        self.WriteI2C(self.devAddr, reg_addr, values)

    def ReadReg(self, reg_addr, reg_len=1):
        return self.ReadI2C(self.devAddr, reg_addr, reg_len)

    def Ind_Acc_Write(self, ia_sel, ind_reg_addr, data, xfer_length=0):
        # range check ia_sel
        if (self.fw.serdes_type == flexrio_wrapper.SerDesType.SER):
            if not (0 <= ia_sel <= 0xF):
                raise ValueError('ia_sel parameter must be in range [0, 0xF]')
        else:
            if not (0 <= ia_sel <= 0x7):
                raise ValueError('ia_sel parameter must be in range [0, 0x7]')

        # range check ind_reg_addr
        if not (0 <= ind_reg_addr <= 0xFF):
            raise ValueError('ind_reg_addr parameter must be in range [0, 0xFF]')

        # range check data and convert data to byte array
        bytes_to_write = bytearray()
        if isinstance(data, list) or isinstance(data, bytearray):
            if (len(data) > 0):
                bytes_to_write.extend(data)
            else:
                raise ValueError('data parameter must have at least one value')
        else:
            bytes_to_write.append(data)

        # configure the IND_ACC_CTL register (0xB0)
        ind_acc_ctl = 0

        if len(bytes_to_write) > 1:
            # set the IA_AUTO_INC field
            ind_acc_ctl |= 0x2

        ind_acc_ctl |= (ia_sel << 2)
        self.WriteReg(0xB0, ind_acc_ctl)

        # configure the IND_ACC_ADDR register (0xB1)
        ind_acc_addr = ind_reg_addr
        self.WriteReg(0xB1, ind_acc_addr)

        # configure the IND_ACC_DATA register (0xB2)
        for byte in bytes_to_write:
            self.WriteReg(0xB2, byte)
